#include "parallel.h"

void main()
{
   char             *Subroutine_name = "node_main():";
   int               Command;
   matrix_node_bin   Matrix;
   vector_node_bin   Vector;

   Dlu_info         *LU_Info;

   tbx_timer        *Timer;
   ZIP_MAILER       *Mailer;


   initialize_node(&Matrix, &Vector);

   Mailer = Matrix.Mailer;
   Zip_push(Mailer);

   Timer = new_tbx_timer();

   receive_matrix_node(Mailer, &Matrix);
   initialize_node_matrix_info(&Matrix, &LU_Info);
   initialize_node_vector_info(&Matrix, &Vector);
   unpack_unfactorized_matrix(&Matrix);
   print("Start LU factor\n");
   G2_sync();
   /* start timer */
   tbx_timer_start(Timer);

   CALL_METHOD(Matrix.Factorization)(LU_Info, NULL, NULL,
                                     XTRA(Matrix.Factorization));
   /* stop timer */
   tbx_timer_stop(Timer);

   /* print time */
   compute_time(Timer);
   transmit_timing_node(Mailer, Timer);
   tbx_timer_reset(Timer);

   receive_col_vector_node(Mailer, &Vector);

   print("Start Solve\n");
   G2_sync();
   /* start timer */
   tbx_timer_start(Timer);

   CALL_METHOD(Matrix.Solve)(LU_Info, Vector.b, Vector.x,
                             XTRA(Matrix.Solve));
   /* stop timer */
   tbx_timer_stop(Timer);

   transmit_row_vector_node(Mailer, &Vector);

   /* print time */
   compute_time(Timer);

#ifdef DEBUG_main_node
   print_time(Timer);
#endif

   transmit_timing_node(Mailer, Timer);
   tbx_timer_reset(Timer);

   Zip_pop();
   zip_exit();
}
